// FletchBot.js
//
// Author:	Poundboy
// Version:	1.02
//
//
// Modification History:
//
// Who	  Date	    Version	Description
//
// PB	  09/28/03  1.01	Initial version.
//
//
// Description:
//
//	Makes greater and deadly arrow heads.  Requires raw materials to be on character
//

FletchDebug	= false;

typePhysical	= 1;
typeElemental	= 2;

function FletchRec(Name, Type, Pea) {
    this.Name		= Name;			//
    this.Type		= Type;			//
    this.Pea		= Pea;			//

    // Info to make elemental
    if (Type == typeElemental) {
	this.Oil	= "Concentrated " + Name + " Oil";
	this.Infuse	= "Concentrated " + Name + " Infusion";
	this.PeaName	= Pea + " Pea";		//
    } else {
    	this.Oil	= null;
    	this.Infuse	= null;
    	this.PeaName	= null;
    } // endif

    this.RawHead	= "Wrapped Bundle of " + Name + " Arrowheads";
    this.GreaterHead	= "Wrapped Bundle of Greater " + Name + " Arrowheads";
    this.DeadlyHead	= "Wrapped Bundle of Deadly " + Name + " Arrowheads";
} // end func

function FletchObj() {

    this.ArrowList	= new Array();

    this.RawElemHead	= "Wrapped Bundle of Arrowheads";

    this.GreaterOil	= "Concentrated Bloodseeker Oil";
    this.GreaterInfuse	= "Concentrated Bloodseeker Infusion";
    this.GreaterPea	= "Quicksilver Pea";

    this.DeadlyOil	= "Concentrated Bloodhunter Oil";
    this.DeadlyInfuse	= "Concentrated Bloodhunter Infusion";
    this.DeadlyPea	= "Stibnite Pea";

    this.ActionComplete = false;
    this.InventAdd	= true;
    this.InventAdjust	= true;

    this.LastTurn	 = new Date();

    this.AddAco		 = null;	// Last ACO created from the AddInventory event
    this.AdjustAco 	 = null;

    // Methods
    this.Init		= Fletch_Init;
    this.CheckTurn	= Fletch_CheckTurn;
    this.MoveItem	= Fletch_MoveItem;
    this.MakeInfusion	= Fletch_MakeInfusion;
    this.MakeOil	= Fletch_MakeOil;
    this.MakeElemental	= Fletch_MakeElemental;
    this.MakeGreater	= Fletch_MakeGreater;
    this.MakeDeadly	= Fletch_MakeDeadly;
} //end func

function Fletch_Init() {
    this.ArrowList[0] = new FletchRec("Blunt",		typePhysical,	null		);
    this.ArrowList[1] = new FletchRec("Armor Piercing",	typePhysical,	null		);
    this.ArrowList[2] = new FletchRec("Frog Crotch",	typePhysical,	null		);
    this.ArrowList[3] = new FletchRec("Fire",		typeElemental,	"Turpeth"	);
    this.ArrowList[4] = new FletchRec("Frost",		typeElemental,	"Colcothar"	);
    this.ArrowList[5] = new FletchRec("Lightning",	typeElemental,	"Cobalt"	);
    this.ArrowList[6] = new FletchRec("Acid",		typeElemental,	"Brimstone"	);
} // end func

function Fletch_CheckTurn() {
    if ((new Date()-this.LastTurn) > (12*60*1000)) {
	skapi.Keys(cmidTurnLeft, kmoDown);
	skapi.Sleep(250);
	skapi.Keys(cmidTurnLeft, kmoUp);
	this.LastTurn = new Date();
    } //endif
} // end func

function Fletch_MoveItem(Aco) {
    if ((Aco == null) || (!(Aco.fExists))) return false;  // If item doesn't exist anymore then abort move
    if (!Aco.fInInventory) return false;		  // If item not in our pack anymore then abort move

    // Clear flag that when set indicates move done
    this.InventAdd    = false;
    this.InventAdjust = false;

    for (var x=0; x<3; x++) {
	// Set timer so we know when to give up
	var LootMoveStart = new Date();

	// Move item to top of main pack
	Aco.MoveToPack(0, 0, true);

	// Wait until we receive the AddToEventory event or the timeout interval has been reached
	while (!this.InventAdd && !this.InventAdjust && ((new Date()-LootMoveStart )<4000)) skapi.WaitEvent(20);

	// If loot moved successfully then return true
	if (this.InventAdd || this.InventAdjust) return true;
    } // endfor

    // If we make it here, then move failed
    return false;
} // end function

function Fletch_MakeInfusion(PeaName) {
    // Find a pea of this type
    var PeaAco = skapi.AcoFindInInv(PeaName);
    if (!PeaAco) {
    	if (FletchDebug) Chat.Screen("MakeInfusion: End, No more peas [" + PeaName + "]");
    	return null;
    } //endif

    this.ActionComplete	= false;
    this.InventAdd	= false;
    var ActionStart     = new Date();

    AlembicAco.UseOnAco(PeaAco);

    while (!(this.ActionComplete && this.InventAdd) && ((new Date()-ActionStart) < 5000))
	skapi.WaitEvent(100, wemFullTimeout);

    if (this.InventAdd) return this.AddAco;
    return null;
} // endif

function Fletch_MakeOil(Infusion, Pea) {
    // Make an oil
    var AquaAco = skapi.AcoFindInInv("Concentrated Aqua Incanta");
    if (!AquaAco) {
	if (FletchDebug) Chat.Screen("MakeOil: End, can't make oil, out of aqua incanta.");
	return null;
    } // endif

    var InfuseAco = skapi.AcoFindInInv(Infusion);
    if (!InfuseAco) {
    	if (FletchDebug) Chat.Screen("MakeOil: No infusion found [" + Infusion + "], trying to make one...");
    	InfuseAco = this.MakeInfusion(Pea);
	if (!InfuseAco) {
	    if (FletchDebug) Chat.Screen("MakeOil: End, can't make infusion");
	    return null;
	} // endif
    } // endif

    this.ActionComplete = false;
    this.InventAdd	= false;
    var ActionStart	= new Date();

    InfuseAco.UseOnAco(AquaAco);

    while (!(this.ActionComplete && this.InventAdd) && ((new Date()-ActionStart) < 5000))
	skapi.WaitEvent(100, wemFullTimeout);

    if (this.InventAdd) return this.AddAco;
    return null;
} // end func

function Fletch_MakeElemental(ArrowType) {
    var ArrowAco = skapi.AcoFindInInv(this.RawElemHead);
    if (!ArrowAco) {
    	if (FletchDebug) Chat.Screen("MakeElemental: Out of raw heads for elemental arrows");
    	return null;
    } // endif

    var OilAco = skapi.AcoFindInInv(this.ArrowList[ArrowType].Oil);
    if (!OilAco) {
    	if (FletchDebug)
	    Chat.Screen("MakeElemental: Out of oil [" + this.ArrowList[ArrowType].Oil + "] trying to make one...");
	OilAco = this.MakeOil(this.ArrowList[ArrowType].Infuse, this.ArrowList[ArrowType].PeaName);
	if (!OilAco) {
	    if (FletchDebug) Chat.Screen("MakeElemental: End, Can't make oil");
	    return null;
	} // endif
    } // endif

    this.ActionComplete = false;
    this.InventAdd	= false;
    var ActionStart	= new Date();

    OilAco.UseOnAco(ArrowAco);

    while (!(this.ActionComplete && this.InventAdd) && ((new Date()-ActionStart) < 5000))
	skapi.WaitEvent(100, wemFullTimeout);

    if (this.InventAdd) return this.AddAco;
    return null;
} // end func

// Convert normal arrow head into greater
function Fletch_MakeGreater(ArrowType) {
    var ArrowAco = skapi.AcoFindInInv(this.ArrowList[ArrowType].RawHead);
    if (!ArrowAco) {
    	// No arrows of the type we need.  If this is an elemental arrow then we can try to create one
    	if (FletchDebug) Chat.Screen("MakeGreater: No head found [" + this.ArrowList[ArrowType].RawHead + "]");
	if (this.ArrowList[ArrowType].Type == typeElemental) {
	    if (FletchDebug) Chat.Screen("MakeGreater: Elemental head, trying to make one");
	    ArrowAco = this.MakeElemental(ArrowType);
	    if (!ArrowAco) {
	    	if (FletchDebug) Chat.Screen("MakeGreater: End, Elemental head not made");
		return null;
	    } // endif
	    if (FletchDebug) Chat.Screen("MakeGreater: Elemental head made");
	} else {
	    return null;
	} // endif
    } // endif

    var GreaterOilAco = skapi.AcoFindInInv(this.GreaterOil);
    if (!GreaterOilAco) {
    	if (FletchDebug) Chat.Screen("MakeGreater: No greater oil found, trying to make one");
    	GreaterOilAco = this.MakeOil(this.GreaterInfuse, this.GreaterPea);
    	if (!GreaterOilAco) {
    	    if (FletchDebug) Chat.Screen("MakeGreater: End, couldn't make oil");
	    return null;
	} // endif
    } // endif

    this.ActionComplete = false;
    this.InventAdd = false;
    ActionStart = new Date();

    GreaterOilAco.UseOnAco(ArrowAco);

    while (!(this.ActionComplete && this.InventAdd) && ((new Date()-ActionStart) < 5000))
	skapi.WaitEvent(100, wemFullTimeout);

    if (this.InventAdd) return this.AddAco;
    return null;
} // end func

function Fletch_MakeDeadly(ArrowType) {
    if (FletchDebug) Chat.Screen("Making deadly " + this.ArrowList[ArrowType].Name + " arrowheads...");

    // Get a greater to convert to a deadly
    var GreaterArrowAco = skapi.AcoFindInInv(this.ArrowList[ArrowType].GreaterHead);

    // If no greater head then make one.
    var NewGreater = false;
    if (!GreaterArrowAco) {
    	if (FletchDebug) Chat.Screen("MakeDeadly: No greater found, trying to make one");
	GreaterArrowAco = this.MakeGreater(ArrowType);
	if (!GreaterArrowAco) {
	    // Can't make a greater for some reason
	    if (FletchDebug) Chat.Screen("MakeDeadly: Can't make greater, done with this type");
	    return false;
	} // endif
	NewGreater = true;
    } //endif

    // We have a greater, now try to convert it to a deadly
    var DeadlyOilAco = skapi.AcoFindInInv(this.DeadlyOil);
    if (!DeadlyOilAco) {
    	if (FletchDebug) Chat.Screen("MakeDeadly: No deadly oil found, trying to make one");
	DeadlyOilAco = this.MakeOil(this.DeadlyInfuse, this.DeadlyPea);
	if (!DeadlyOilAco) {
	    // Can't convert to deadly. If this is a new greater then combine it with others of same type.
	    if (FletchDebug) Chat.Screen("MakeDeadly: Can't make deadly oil");
	    if (NewGreater) {
	    	if (FletchDebug) Chat.Screen("MakeDeadly: New greater, combining...");
	    	this.MoveItem(GreaterArrowAco);
	    } else {
	    	// Pre-existing greater, try to create another one and combine it with others like it.
	    	if (FletchDebug) Chat.Screen("MakeDeadly: Old greater, trying to make one");
	    	GreaterArrowAco = this.MakeGreater(ArrowType);
	    	if (GreaterArrowAco) this.MoveItem(GreaterArrowAco);
	    } // endif
	    return;
	} // endif
    } // endif

    this.ActionComplete = false;
    this.InventAdd	= false;
    ActionStart		= new Date();

    DeadlyOilAco.UseOnAco(GreaterArrowAco);

    while (!(this.ActionComplete && this.InventAdd) && ((new Date()-ActionStart) < 5000))
	skapi.WaitEvent(100, wemFullTimeout);

    if (this.InventAdd) {
	// Success, combine new deadly with others of same type
	this.MoveItem(this.AddAco);
    } else {
	// Failure, combine greater aco with others of same type
	this.MoveItem(GreaterArrowAco);
    } // endif

    return true;
} // endfunc


// Event functions

// OnActionComplete
function Fletch_OnActionComplete() {
    Fletch.ActionComplete = true;
    return;
} // end funct

function Fletch_OnAddToInventory(aco) {
    Fletch.InventAdd = true;
    Fletch.AddAco = aco;
    return;
} // end func

function Fletch_OnAdjustStack(aco,citemPrev) {
    Fletch.InventAdjust = true;
    Fletch.AdjustAco = aco;
    return;
} // end func

function Fletch_Handlers() {
	this.OnActionComplete	  	= Fletch_OnActionComplete;
	this.OnAddToInventory	 	= Fletch_OnAddToInventory;
	this.OnAdjustStack	  	= Fletch_OnAdjustStack;
} // end func

function Fletch_AddHandlers() {
    // Active handlers
    skapi.AddHandler(evidOnActionComplete,	FletchH );
    skapi.AddHandler(evidOnAddToInventory,	FletchH );
    skapi.AddHandler(evidOnAdjustStack,		FletchH );
} // end func

function Fletch_RemoveHandlers() {
    // Active handlers
    skapi.RemoveHandler(evidOnActionComplete,	FletchH );
    skapi.RemoveHandler(evidOnAddToInventory,	FletchH );
    skapi.RemoveHandler(evidOnAdjustStack,	FletchH );
} // end func

// Define fletch object
Fletch	= new FletchObj();	// Define the fletch object

// Event handlers
FletchH = new Fletch_Handlers();


function main() {

    Chat.Screen("Begin");

    AlembicAco = skapi.AcoFindInInv("Alembic");
    if (AlembicAco == null) {
    	Chat.Screen("No Alembic in inventory!");
    	return;
    } // endif

    Fletch_AddHandlers();
    Fletch.Init();

    while (true) {

    	skapi.WaitEvent(100, wemFullTimeout);

	// Make one arrow of each type in round-robin order
	for (var x=0; x<Fletch.ArrowList.length; x++) Fletch.MakeDeadly(x);

	Fletch.CheckTurn();
    } // endif

} // end func